@@ -4,23 +4,32 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
-#
-# $Id: asf.py 4224 2007-12-03 09:01:49Z luks $
 
 """Read and write ASF (Window Media Audio) files."""
 
 __all__ = ["ASF", "Open"]
 
+import sys
 import struct
-from mutagen import FileType, Metadata
-from mutagen._util import insert_bytes, delete_bytes, DictMixin
-
-class error(IOError): pass
-class ASFError(error): pass
-class ASFHeaderError(error): pass
+from mutagen import FileType, Metadata, StreamInfo
+from mutagen._util import insert_bytes, delete_bytes, DictMixin, \
+    total_ordering, MutagenError
+from ._compat import swap_to_string, text_type, PY2, string_types, reraise
 
 
-class ASFInfo(object):
+class error(IOError, MutagenError):
+    pass
+
+
+class ASFError(error):
+    pass
+
+
+class ASFHeaderError(error):
+    pass
+
+
+class ASFInfo(StreamInfo):
     """ASF stream information."""
 
     def __init__(self):
@@ -49,20 +58,27 @@ class ASFTags(list, DictMixin, Metadata):
 
         """
         values = [value for (k, value) in self if k == key]
-        if not values: raise KeyError, key
-        else: return values
+        if not values:
+            raise KeyError(key)
+        else:
+            return values
 
     def __delitem__(self, key):
         """Delete all values associated with the key."""
-        to_delete = filter(lambda x: x[0] == key, self)
-        if not to_delete: raise KeyError, key
-        else: map(self.remove, to_delete)
+        to_delete = list(filter(lambda x: x[0] == key, self))
+        if not to_delete:
+            raise KeyError(key)
+        else:
+            for k in to_delete:
+                self.remove(k)
 
     def __contains__(self, key):
         """Return true if the key has any values."""
         for k, value in self:
-            if k == key: return True
-        else: return False
+            if k == key:
+                return True
+        else:
+            return False
 
     def __setitem__(self, key, values):
         """Set a key's value or values.
@@ -74,14 +90,17 @@ class ASFTags(list, DictMixin, Metadata):
         """
         if not isinstance(values, list):
             values = [values]
-        try: del(self[key])
-        except KeyError: pass
+        try:
+            del(self[key])
+        except KeyError:
+            pass
         for value in values:
             if key in _standard_attribute_names:
-                value = unicode(value)
+                value = text_type(value)
             elif not isinstance(value, ASFBaseAttribute):
-                if isinstance(value, basestring):
-                    value = ASFUnicodeAttribute(value)
+                if isinstance(value, string_types):
+                    if PY2 or isinstance(value, text_type):
+                        value = ASFUnicodeAttribute(value)
                 elif isinstance(value, bool):
                     value = ASFBoolAttribute(value)
                 elif isinstance(value, int):
@@ -92,7 +111,7 @@ class ASFTags(list, DictMixin, Metadata):
 
     def keys(self):
         """Return all keys in the comment."""
-        return self and set(zip(*self)[0])
+        return self and set(next(iter(zip(*self))))
 
     def as_dict(self):
         """Return a copy of the comment data in a real dict."""
@@ -113,7 +132,19 @@ class ASFBaseAttribute(object):
         if data:
             self.value = self.parse(data, **kwargs)
         else:
-            self.value = value
+            if value is None:
+                # we used to support not passing any args and instead assign
+                # them later, keep that working..
+                self.value = None
+            else:
+                self.value = self._validate(value)
+
+    def _validate(self, value):
+        """Raises TypeError or ValueError in case the user supplied value
+        isn't valid.
+        """
+
+        return value
 
     def data_size(self):
         raise NotImplementedError
@@ -128,13 +159,13 @@ class ASFBaseAttribute(object):
         return name
 
     def render(self, name):
-        name = name.encode("utf-16-le") + "\x00\x00"
+        name = name.encode("utf-16-le") + b"\x00\x00"
         data = self._render()
         return (struct.pack("<H", len(name)) + name +
                 struct.pack("<HH", self.TYPE, len(data)) + data)
 
     def render_m(self, name):
-        name = name.encode("utf-16-le") + "\x00\x00"
+        name = name.encode("utf-16-le") + b"\x00\x00"
         if self.TYPE == 2:
             data = self._render(dword=False)
         else:
@@ -143,58 +174,93 @@ class ASFBaseAttribute(object):
                             self.TYPE, len(data)) + name + data)
 
     def render_ml(self, name):
-        name = name.encode("utf-16-le") + "\x00\x00"
+        name = name.encode("utf-16-le") + b"\x00\x00"
         if self.TYPE == 2:
             data = self._render(dword=False)
         else:
             data = self._render()
+
         return (struct.pack("<HHHHI", self.language or 0, self.stream or 0,
                             len(name), self.TYPE, len(data)) + name + data)
 
+
+@swap_to_string
+@total_ordering
 class ASFUnicodeAttribute(ASFBaseAttribute):
     """Unicode string attribute."""
     TYPE = 0x0000
 
     def parse(self, data):
-        return data.decode("utf-16-le").strip("\x00")
+        try:
+            return data.decode("utf-16-le").strip("\x00")
+        except UnicodeDecodeError as e:
+            reraise(ASFError, e, sys.exc_info()[2])
+
+    def _validate(self, value):
+        if not isinstance(value, text_type):
+            if PY2:
+                return value.decode("utf-8")
+            else:
+                raise TypeError("%r not str" % value)
+        return value
 
     def _render(self):
-        return self.value.encode("utf-16-le") + "\x00\x00"
+        return self.value.encode("utf-16-le") + b"\x00\x00"
 
     def data_size(self):
         return len(self._render())
 
+    def __bytes__(self):
+        return self.value.encode("utf-16-le")
+
     def __str__(self):
         return self.value
 
-    def __cmp__(self, other):
-        return cmp(unicode(self), other)
+    def __eq__(self, other):
+        return text_type(self) == other
+
+    def __lt__(self, other):
+        return text_type(self) < other
 
     __hash__ = ASFBaseAttribute.__hash__
 
 
+@swap_to_string
+@total_ordering
 class ASFByteArrayAttribute(ASFBaseAttribute):
     """Byte array attribute."""
     TYPE = 0x0001
 
     def parse(self, data):
+        assert isinstance(data, bytes)
         return data
 
     def _render(self):
+        assert isinstance(self.value, bytes)
         return self.value
 
+    def _validate(self, value):
+        if not isinstance(value, bytes):
+            raise TypeError("must be bytes/str: %r" % value)
+        return value
+
     def data_size(self):
         return len(self.value)
 
-    def __str__(self):
+    def __bytes__(self):
         return "[binary data (%s bytes)]" % len(self.value)
 
-    def __cmp__(self, other):
-        return cmp(str(self), other)
+    def __eq__(self, other):
+        return self.value == other
+
+    def __lt__(self, other):
+        return self.value < other
 
     __hash__ = ASFBaseAttribute.__hash__
 
 
+@swap_to_string
+@total_ordering
 class ASFBoolAttribute(ASFBaseAttribute):
     """Bool attribute."""
     TYPE = 0x0002
@@ -207,25 +273,33 @@ class ASFBoolAttribute(ASFBaseAttribute):
 
     def _render(self, dword=True):
         if dword:
-            return struct.pack("<I", int(self.value))
+            return struct.pack("<I", bool(self.value))
         else:
-            return struct.pack("<H", int(self.value))
+            return struct.pack("<H", bool(self.value))
+
+    def _validate(self, value):
+        return bool(value)
 
     def data_size(self):
         return 4
 
     def __bool__(self):
-        return self.value
+        return bool(self.value)
 
-    def __str__(self):
-        return str(self.value)
+    def __bytes__(self):
+        return text_type(self.value).encode('utf-8')
 
-    def __cmp__(self, other):
-        return cmp(bool(self), other)
+    def __eq__(self, other):
+        return bool(self.value) == other
+
+    def __lt__(self, other):
+        return bool(self.value) < other
 
     __hash__ = ASFBaseAttribute.__hash__
 
 
+@swap_to_string
+@total_ordering
 class ASFDWordAttribute(ASFBaseAttribute):
     """DWORD attribute."""
     TYPE = 0x0003
@@ -236,21 +310,32 @@ class ASFDWordAttribute(ASFBaseAttribute):
     def _render(self):
         return struct.pack("<L", self.value)
 
+    def _validate(self, value):
+        value = int(value)
+        if not 0 <= value <= 2 ** 32 - 1:
+            raise ValueError("Out of range")
+        return value
+
     def data_size(self):
         return 4
 
     def __int__(self):
         return self.value
 
-    def __str__(self):
-        return str(self.value)
+    def __bytes__(self):
+        return self.value
 
-    def __cmp__(self, other):
-        return cmp(int(self), other)
+    def __eq__(self, other):
+        return int(self.value) == other
+
+    def __lt__(self, other):
+        return int(self.value) < other
 
     __hash__ = ASFBaseAttribute.__hash__
 
 
+@swap_to_string
+@total_ordering
 class ASFQWordAttribute(ASFBaseAttribute):
     """QWORD attribute."""
     TYPE = 0x0004
@@ -261,21 +346,32 @@ class ASFQWordAttribute(ASFBaseAttribute):
     def _render(self):
         return struct.pack("<Q", self.value)
 
+    def _validate(self, value):
+        value = int(value)
+        if not 0 <= value <= 2 ** 64 - 1:
+            raise ValueError("Out of range")
+        return value
+
     def data_size(self):
         return 8
 
     def __int__(self):
         return self.value
 
-    def __str__(self):
-        return str(self.value)
+    def __bytes__(self):
+        return self.value
 
-    def __cmp__(self, other):
-        return cmp(int(self), other)
+    def __eq__(self, other):
+        return int(self.value) == other
+
+    def __lt__(self, other):
+        return int(self.value) < other
 
     __hash__ = ASFBaseAttribute.__hash__
 
 
+@swap_to_string
+@total_ordering
 class ASFWordAttribute(ASFBaseAttribute):
     """WORD attribute."""
     TYPE = 0x0005
@@ -286,39 +382,60 @@ class ASFWordAttribute(ASFBaseAttribute):
     def _render(self):
         return struct.pack("<H", self.value)
 
+    def _validate(self, value):
+        value = int(value)
+        if not 0 <= value <= 2 ** 16 - 1:
+            raise ValueError("Out of range")
+        return value
+
     def data_size(self):
         return 2
 
     def __int__(self):
         return self.value
 
-    def __str__(self):
-        return str(self.value)
+    def __bytes__(self):
+        return self.value
 
-    def __cmp__(self, other):
-        return cmp(int(self), other)
+    def __eq__(self, other):
+        return int(self.value) == other
+
+    def __lt__(self, other):
+        return int(self.value) < other
 
     __hash__ = ASFBaseAttribute.__hash__
 
 
+@swap_to_string
+@total_ordering
 class ASFGUIDAttribute(ASFBaseAttribute):
     """GUID attribute."""
     TYPE = 0x0006
 
     def parse(self, data):
+        assert isinstance(data, bytes)
         return data
 
     def _render(self):
+        assert isinstance(self.value, bytes)
         return self.value
 
+    def _validate(self, value):
+        if not isinstance(value, bytes):
+            raise TypeError("must be bytes/str: %r" % value)
+        return value
+
     def data_size(self):
         return len(self.value)
 
-    def __str__(self):
+    def __bytes__(self):
         return self.value
 
-    def __cmp__(self, other):
-        return cmp(str(self), other)
+    def __eq__(self, other):
+        return self.value == other
+
+    def __lt__(self, other):
+        return self.value < other
 
     __hash__ = ASFBaseAttribute.__hash__
 
@@ -331,6 +448,7 @@ QWORD = ASFQWordAttribute.TYPE
 WORD = ASFWordAttribute.TYPE
 GUID = ASFGUIDAttribute.TYPE
 
+
 def ASFValue(value, kind, **kwargs):
     for t, c in _attribute_types.items():
         if kind == t:
@@ -367,24 +485,24 @@ class BaseObject(object):
 
     def render(self, asf):
         data = self.GUID + struct.pack("<Q", len(self.data) + 24) + self.data
-        size = len(data)
         return data
 
 
 class UnknownObject(BaseObject):
     """Unknown ASF object."""
     def __init__(self, guid):
+        assert isinstance(guid, bytes)
         self.GUID = guid
 
 
 class HeaderObject(object):
     """ASF header."""
-    GUID = "\x30\x26\xB2\x75\x8E\x66\xCF\x11\xA6\xD9\x00\xAA\x00\x62\xCE\x6C"
+    GUID = b"\x30\x26\xB2\x75\x8E\x66\xCF\x11\xA6\xD9\x00\xAA\x00\x62\xCE\x6C"
 
 
 class ContentDescriptionObject(BaseObject):
     """Content description."""
-    GUID = "\x33\x26\xB2\x75\x8E\x66\xCF\x11\xA6\xD9\x00\xAA\x00\x62\xCE\x6C"
+    GUID = b"\x33\x26\xB2\x75\x8E\x66\xCF\x11\xA6\xD9\x00\xAA\x00\x62\xCE\x6C"
 
     def parse(self, asf, data, fileobj, size):
         super(ContentDescriptionObject, self).parse(asf, data, fileobj, size)
@@ -405,7 +523,8 @@ class ContentDescriptionObject(BaseObject):
             Author=author,
             Copyright=copyright,
             Description=desc,
-            Rating=rating).items():
+            Rating=rating
+        ).items():
             if value is not None:
                 asf.tags[key] = value
 
@@ -413,45 +532,47 @@ class ContentDescriptionObject(BaseObject):
         def render_text(name):
             value = asf.tags.get(name, [])
             if value:
-                return value[0].encode("utf-16-le") + "\x00\x00"
+                return value[0].encode("utf-16-le") + b"\x00\x00"
             else:
-                return ""
-        texts = map(render_text, _standard_attribute_names)
-        data = struct.pack("<HHHHH", *map(len, texts)) + "".join(texts)
+                return b""
+        texts = list(map(render_text, _standard_attribute_names))
+        data = struct.pack("<HHHHH", *map(len, texts)) + b"".join(texts)
         return self.GUID + struct.pack("<Q", 24 + len(data)) + data
 
 
 class ExtendedContentDescriptionObject(BaseObject):
     """Extended content description."""
-    GUID = "\x40\xA4\xD0\xD2\x07\xE3\xD2\x11\x97\xF0\x00\xA0\xC9\x5E\xA8\x50"
+    GUID = b"\x40\xA4\xD0\xD2\x07\xE3\xD2\x11\x97\xF0\x00\xA0\xC9\x5E\xA8\x50"
 
     def parse(self, asf, data, fileobj, size):
-        super(ExtendedContentDescriptionObject, self).parse(asf, data, fileobj, size)
+        super(ExtendedContentDescriptionObject, self).parse(
+            asf, data, fileobj, size)
         asf.extended_content_description_obj = self
         num_attributes, = struct.unpack("<H", data[0:2])
         pos = 2
         for i in range(num_attributes):
-            name_length, = struct.unpack("<H", data[pos:pos+2])
+            name_length, = struct.unpack("<H", data[pos:pos + 2])
             pos += 2
-            name = data[pos:pos+name_length].decode("utf-16-le").strip("\x00")
+            name = data[pos:pos + name_length]
+            name = name.decode("utf-16-le").strip("\x00")
             pos += name_length
-            value_type, value_length = struct.unpack("<HH", data[pos:pos+4])
+            value_type, value_length = struct.unpack("<HH", data[pos:pos + 4])
             pos += 4
-            value = data[pos:pos+value_length]
+            value = data[pos:pos + value_length]
             pos += value_length
             attr = _attribute_types[value_type](data=value)
             asf.tags.append((name, attr))
 
     def render(self, asf):
         attrs = asf.to_extended_content_description.items()
-        data = "".join([attr.render(name) for (name, attr) in attrs])
+        data = b"".join([attr.render(name) for (name, attr) in attrs])
         data = struct.pack("<QH", 26 + len(data), len(attrs)) + data
         return self.GUID + data
 
 
 class FilePropertiesObject(BaseObject):
     """File properties."""
-    GUID = "\xA1\xDC\xAB\x8C\x47\xA9\xCF\x11\x8E\xE4\x00\xC0\x0C\x20\x53\x65"
+    GUID = b"\xA1\xDC\xAB\x8C\x47\xA9\xCF\x11\x8E\xE4\x00\xC0\x0C\x20\x53\x65"
 
     def parse(self, asf, data, fileobj, size):
         super(FilePropertiesObject, self).parse(asf, data, fileobj, size)
@@ -461,7 +582,7 @@ class FilePropertiesObject(BaseObject):
 
 class StreamPropertiesObject(BaseObject):
     """Stream properties."""
-    GUID = "\x91\x07\xDC\xB7\xB7\xA9\xCF\x11\x8E\xE6\x00\xC0\x0C\x20\x53\x65"
+    GUID = b"\x91\x07\xDC\xB7\xB7\xA9\xCF\x11\x8E\xE6\x00\xC0\x0C\x20\x53\x65"
 
     def parse(self, asf, data, fileobj, size):
         super(StreamPropertiesObject, self).parse(asf, data, fileobj, size)
@@ -473,7 +594,7 @@ class StreamPropertiesObject(BaseObject):
 
 class HeaderExtensionObject(BaseObject):
     """Header extension."""
-    GUID = "\xb5\x03\xbf_.\xa9\xcf\x11\x8e\xe3\x00\xc0\x0c Se"
+    GUID = b"\xb5\x03\xbf_.\xa9\xcf\x11\x8e\xe3\x00\xc0\x0c Se"
 
     def parse(self, asf, data, fileobj, size):
         super(HeaderExtensionObject, self).parse(asf, data, fileobj, size)
@@ -482,26 +603,28 @@ class HeaderExtensionObject(BaseObject):
         datapos = 0
         self.objects = []
         while datapos < datasize:
-            guid, size = struct.unpack("<16sQ", data[22+datapos:22+datapos+24])
+            guid, size = struct.unpack(
+                "<16sQ", data[22 + datapos:22 + datapos + 24])
             if guid in _object_types:
                 obj = _object_types[guid]()
             else:
                 obj = UnknownObject(guid)
-            obj.parse(asf, data[22+datapos+24:22+datapos+size], fileobj, size)
+            obj.parse(asf, data[22 + datapos + 24:22 + datapos + size],
+                      fileobj, size)
             self.objects.append(obj)
             datapos += size
 
     def render(self, asf):
-        data = "".join([obj.render(asf) for obj in self.objects])
+        data = b"".join([obj.render(asf) for obj in self.objects])
         return (self.GUID + struct.pack("<Q", 24 + 16 + 6 + len(data)) +
-                "\x11\xD2\xD3\xAB\xBA\xA9\xcf\x11" +
-                "\x8E\xE6\x00\xC0\x0C\x20\x53\x65" +
-                "\x06\x00" + struct.pack("<I", len(data)) + data)
+                b"\x11\xD2\xD3\xAB\xBA\xA9\xcf\x11" +
+                b"\x8E\xE6\x00\xC0\x0C\x20\x53\x65" +
+                b"\x06\x00" + struct.pack("<I", len(data)) + data)
 
 
 class MetadataObject(BaseObject):
     """Metadata description."""
-    GUID = "\xea\xcb\xf8\xc5\xaf[wH\x84g\xaa\x8cD\xfaL\xca"
+    GUID = b"\xea\xcb\xf8\xc5\xaf[wH\x84g\xaa\x8cD\xfaL\xca"
 
     def parse(self, asf, data, fileobj, size):
         super(MetadataObject, self).parse(asf, data, fileobj, size)
@@ -510,11 +633,12 @@ class MetadataObject(BaseObject):
         pos = 2
         for i in range(num_attributes):
             (reserved, stream, name_length, value_type,
-             value_length) = struct.unpack("<HHHHI", data[pos:pos+12])
+             value_length) = struct.unpack("<HHHHI", data[pos:pos + 12])
             pos += 12
-            name = data[pos:pos+name_length].decode("utf-16-le").strip("\x00")
+            name = data[pos:pos + name_length]
+            name = name.decode("utf-16-le").strip("\x00")
             pos += name_length
-            value = data[pos:pos+value_length]
+            value = data[pos:pos + value_length]
             pos += value_length
             args = {'data': value, 'stream': stream}
             if value_type == 2:
@@ -524,14 +648,14 @@ class MetadataObject(BaseObject):
 
     def render(self, asf):
         attrs = asf.to_metadata.items()
-        data = "".join([attr.render_m(name) for (name, attr) in attrs])
+        data = b"".join([attr.render_m(name) for (name, attr) in attrs])
         return (self.GUID + struct.pack("<QH", 26 + len(data), len(attrs)) +
                 data)
 
 
 class MetadataLibraryObject(BaseObject):
     """Metadata library description."""
-    GUID = "\x94\x1c#D\x98\x94\xd1I\xa1A\x1d\x13NEpT"
+    GUID = b"\x94\x1c#D\x98\x94\xd1I\xa1A\x1d\x13NEpT"
 
     def parse(self, asf, data, fileobj, size):
         super(MetadataLibraryObject, self).parse(asf, data, fileobj, size)
@@ -540,11 +664,12 @@ class MetadataLibraryObject(BaseObject):
         pos = 2
         for i in range(num_attributes):
             (language, stream, name_length, value_type,
-             value_length) = struct.unpack("<HHHHI", data[pos:pos+12])
+             value_length) = struct.unpack("<HHHHI", data[pos:pos + 12])
             pos += 12
-            name = data[pos:pos+name_length].decode("utf-16-le").strip("\x00")
+            name = data[pos:pos + name_length]
+            name = name.decode("utf-16-le").strip("\x00")
             pos += name_length
-            value = data[pos:pos+value_length]
+            value = data[pos:pos + value_length]
             pos += value_length
             args = {'data': value, 'language': language, 'stream': stream}
             if value_type == 2:
@@ -554,7 +679,7 @@ class MetadataLibraryObject(BaseObject):
 
     def render(self, asf):
         attrs = asf.to_metadata_library
-        data = "".join([attr.render_ml(name) for (name, attr) in attrs])
+        data = b"".join([attr.render_ml(name) for (name, attr) in attrs])
         return (self.GUID + struct.pack("<QH", 26 + len(data), len(attrs)) +
                 data)
 
@@ -602,8 +727,8 @@ class ASF(FileType):
                 continue
             library_only = (value.data_size() > 0xFFFF or value.TYPE == GUID)
             if (value.language is None and value.stream is None and
-                name not in self.to_extended_content_description and
-                not library_only):
+                    name not in self.to_extended_content_description and
+                    not library_only):
                 self.to_extended_content_description[name] = value
             elif (value.language is None and value.stream is not None and
                   name not in self.to_metadata and not library_only):
@@ -634,10 +759,10 @@ class ASF(FileType):
             self.header_extension_obj.objects.append(self.metadata_library_obj)
 
         # Render the header
-        data = "".join([obj.render(self) for obj in self.objects])
+        data = b"".join([obj.render(self) for obj in self.objects])
         data = (HeaderObject.GUID +
                 struct.pack("<QL", len(data) + 30, len(self.objects)) +
-                "\x01\x02" + data)
+                b"\x01\x02" + data)
 
         fileobj = open(self.filename, "rb+")
         try:
@@ -657,7 +782,7 @@ class ASF(FileType):
     def __read_file(self, fileobj):
         header = fileobj.read(30)
         if len(header) != 30 or header[:16] != HeaderObject.GUID:
-            raise ASFHeaderError, "Not an ASF file."
+            raise ASFHeaderError("Not an ASF file.")
 
         self.extended_content_description_obj = None
         self.content_description_obj = None
@@ -680,8 +805,8 @@ class ASF(FileType):
         obj.parse(self, data, fileobj, size)
         self.objects.append(obj)
 
+    @staticmethod
     def score(filename, fileobj, header):
         return header.startswith(HeaderObject.GUID) * 2
-    score = staticmethod(score)
 
 Open = ASF
