@@ -3,8 +3,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
-#
-# $Id: ogg.py 3975 2007-01-13 21:51:17Z piman $
 
 """Read and write Ogg bitstreams and pages.
 
@@ -20,15 +18,17 @@ import struct
 import sys
 import zlib
 
-from cStringIO import StringIO
-
 from mutagen import FileType
-from mutagen._util import cdata, insert_bytes, delete_bytes
+from mutagen._util import cdata, insert_bytes, delete_bytes, MutagenError
+from ._compat import cBytesIO, reraise, chr_
 
-class error(IOError):
+
+class error(IOError, MutagenError):
     """Ogg stream parsing errors."""
+
     pass
 
+
 class OggPage(object):
     """A single Ogg page (not necessarily a single encoded packet).
 
@@ -40,13 +40,14 @@ class OggPage(object):
     to the start of the next page.
 
     Attributes:
-    version -- stream structure version (currently always 0)
-    position -- absolute stream position (default -1)
-    serial -- logical stream serial number (default 0)
-    sequence -- page sequence number within logical stream (default 0)
-    offset -- offset this page was read from (default None)
-    complete -- if the last packet on this page is complete (default True)
-    packets -- list of raw packet data (default [])
+
+    * version -- stream structure version (currently always 0)
+    * position -- absolute stream position (default -1)
+    * serial -- logical stream serial number (default 0)
+    * sequence -- page sequence number within logical stream (default 0)
+    * offset -- offset this page was read from (default None)
+    * complete -- if the last packet on this page is complete (default True)
+    * packets -- list of raw packet data (default [])
 
     Note that if 'complete' is false, the next page's 'continued'
     property must be true (so set both when constructing pages).
@@ -57,7 +58,7 @@ class OggPage(object):
 
     version = 0
     __type_flags = 0
-    position = 0L
+    position = 0
     serial = 0
     sequence = 0
     offset = None
@@ -78,13 +79,13 @@ class OggPage(object):
         try:
             (oggs, self.version, self.__type_flags, self.position,
              self.serial, self.sequence, crc, segments) = struct.unpack(
-                "<4sBBqIIiB", header)
+                 "<4sBBqIIiB", header)
         except struct.error:
             raise error("unable to read full header; got %r" % header)
 
-        if oggs != "OggS":
+        if oggs != b"OggS":
             raise error("read %r, expected %r, at 0x%x" % (
-                oggs, "OggS", fileobj.tell() - 27))
+                oggs, b"OggS", fileobj.tell() - 27))
 
         if self.version != 0:
             raise error("version %r unsupported" % self.version)
@@ -94,7 +95,7 @@ class OggPage(object):
         lacing_bytes = fileobj.read(segments)
         if len(lacing_bytes) != segments:
             raise error("unable to read %r lacing bytes" % segments)
-        for c in map(ord, lacing_bytes):
+        for c in bytearray(lacing_bytes):
             total += c
             if c < 255:
                 lacings.append(total)
@@ -103,8 +104,8 @@ class OggPage(object):
             lacings.append(total)
             self.complete = False
 
-        self.packets = map(fileobj.read, lacings)
-        if map(len, self.packets) != lacings:
+        self.packets = [fileobj.read(l) for l in lacings]
+        if [len(p) for p in self.packets] != lacings:
             raise error("unable to read full data")
 
     def __eq__(self, other):
@@ -132,21 +133,21 @@ class OggPage(object):
         """
 
         data = [
-            struct.pack("<4sBBqIIi", "OggS", self.version, self.__type_flags,
+            struct.pack("<4sBBqIIi", b"OggS", self.version, self.__type_flags,
                         self.position, self.serial, self.sequence, 0)
-            ]
+        ]
 
         lacing_data = []
         for datum in self.packets:
             quot, rem = divmod(len(datum), 255)
-            lacing_data.append("\xff" * quot + chr(rem))
-        lacing_data = "".join(lacing_data)
-        if not self.complete and lacing_data.endswith("\x00"):
+            lacing_data.append(b"\xff" * quot + chr_(rem))
+        lacing_data = b"".join(lacing_data)
+        if not self.complete and lacing_data.endswith(b"\x00"):
             lacing_data = lacing_data[:-1]
-        data.append(chr(len(lacing_data)))
+        data.append(chr_(len(lacing_data)))
         data.append(lacing_data)
         data.extend(self.packets)
-        data = "".join(data)
+        data = b"".join(data)
 
         # Python's CRC is swapped relative to Ogg's needs.
         # crc32 returns uint prior to py2.6 on some platforms, so force uint
@@ -157,8 +158,11 @@ class OggPage(object):
         data = data[:22] + crc + data[26:]
         return data
 
-    def __size(self):
-        size = 27 # Initial header size
+    @property
+    def size(self):
+        """Total frame size."""
+
+        size = 27  # Initial header size
         for datum in self.packets:
             quot, rem = divmod(len(datum), 255)
             size += quot + 1
@@ -169,12 +173,12 @@ class OggPage(object):
         size += sum(map(len, self.packets))
         return size
 
-    size = property(__size, doc="Total frame size.")
-
     def __set_flag(self, bit, val):
         mask = 1 << bit
-        if val: self.__type_flags |= mask
-        else: self.__type_flags &= ~mask
+        if val:
+            self.__type_flags |= mask
+        else:
+            self.__type_flags &= ~mask
 
     continued = property(
         lambda self: cdata.test_bit(self.__type_flags, 0),
@@ -191,6 +195,7 @@ class OggPage(object):
         lambda self, v: self.__set_flag(2, v),
         doc="This is the last page of a logical bitstream.")
 
+    @classmethod
     def renumber(klass, fileobj, serial, start):
         """Renumber pages belonging to a specified logical stream.
 
@@ -213,7 +218,8 @@ class OggPage(object):
 
         number = start
         while True:
-            try: page = OggPage(fileobj)
+            try:
+                page = OggPage(fileobj)
             except EOFError:
                 break
             else:
@@ -227,8 +233,8 @@ class OggPage(object):
             fileobj.write(page.write())
             fileobj.seek(page.offset + page.size, 0)
             number += 1
-    renumber = classmethod(renumber)
 
+    @classmethod
     def to_packets(klass, pages, strict=False):
         """Construct a list of packet data from a list of Ogg pages.
 
@@ -246,22 +252,25 @@ class OggPage(object):
             if not pages[-1].complete:
                 raise ValueError("last packet does not complete")
         elif pages and pages[0].continued:
-            packets.append("")
+            packets.append([b""])
 
         for page in pages:
             if serial != page.serial:
                 raise ValueError("invalid serial number in %r" % page)
             elif sequence != page.sequence:
                 raise ValueError("bad sequence number in %r" % page)
-            else: sequence += 1
+            else:
+                sequence += 1
 
-            if page.continued: packets[-1] += page.packets[0]
-            else: packets.append(page.packets[0])
-            packets.extend(page.packets[1:])
+            if page.continued:
+                packets[-1].append(page.packets[0])
+            else:
+                packets.append([page.packets[0]])
+            packets.extend([[p] for p in page.packets[1:]])
 
-        return packets
-    to_packets = classmethod(to_packets)
+        return [b"".join(p) for p in packets]
 
+    @classmethod
     def from_packets(klass, packets, sequence=0,
                      default_size=4096, wiggle_room=2048):
         """Construct a list of Ogg pages from a list of packet data.
@@ -290,7 +299,7 @@ class OggPage(object):
         page.sequence = sequence
 
         for packet in packets:
-            page.packets.append("")
+            page.packets.append(b"")
             while packet:
                 data, packet = packet[:chunk_size], packet[chunk_size:]
                 if page.size < default_size and len(page.packets) < 255:
@@ -304,7 +313,7 @@ class OggPage(object):
                     if page.packets[-1]:
                         page.complete = False
                         if len(page.packets) == 1:
-                            page.position = -1L
+                            page.position = -1
                     else:
                         page.packets.pop(-1)
                     pages.append(page)
@@ -315,14 +324,14 @@ class OggPage(object):
 
                 if len(packet) < wiggle_room:
                     page.packets[-1] += packet
-                    packet = ""
+                    packet = b""
 
         if page.packets:
             pages.append(page)
 
         return pages
-    from_packets = classmethod(from_packets)
 
+    @classmethod
     def replace(klass, fileobj, old_pages, new_pages):
         """Replace old_pages with new_pages within fileobj.
 
@@ -349,9 +358,9 @@ class OggPage(object):
         new_pages[-1].last = old_pages[-1].last
         new_pages[-1].complete = old_pages[-1].complete
         if not new_pages[-1].complete and len(new_pages[-1].packets) == 1:
-            new_pages[-1].position = -1L
+            new_pages[-1].position = -1
 
-        new_data = "".join(map(klass.write, new_pages))
+        new_data = b"".join(map(klass.write, new_pages))
 
         # Make room in the file for the new data.
         delta = len(new_data)
@@ -377,8 +386,8 @@ class OggPage(object):
             serial = new_pages[-1].serial
             sequence = new_pages[-1].sequence + 1
             klass.renumber(fileobj, serial, sequence)
-    replace = classmethod(replace)
 
+    @classmethod
     def find_last(klass, fileobj, serial):
         """Find the last page of the stream 'serial'.
 
@@ -390,15 +399,17 @@ class OggPage(object):
         """
 
         # For non-muxed streams, look at the last page.
-        try: fileobj.seek(-256*256, 2)
+        try:
+            fileobj.seek(-256 * 256, 2)
         except IOError:
             # The file is less than 64k in length.
             fileobj.seek(0)
         data = fileobj.read()
-        try: index = data.rindex("OggS")
+        try:
+            index = data.rindex(b"OggS")
         except ValueError:
             raise error("unable to find final Ogg header")
-        stringobj = StringIO(data[index:])
+        stringobj = cBytesIO(data[index:])
         best_page = None
         try:
             page = OggPage(stringobj)
@@ -406,9 +417,12 @@ class OggPage(object):
             pass
         else:
             if page.serial == serial:
-                if page.last: return page
-                else: best_page = page
-            else: best_page = None
+                if page.last:
+                    return page
+                else:
+                    best_page = page
+            else:
+                best_page = None
 
         # The stream is muxed, so use the slow way.
         fileobj.seek(0)
@@ -424,7 +438,7 @@ class OggPage(object):
             return best_page
         except EOFError:
             return best_page
-    find_last = classmethod(find_last)
+
 
 class OggFileType(FileType):
     """An generic Ogg file."""
@@ -444,10 +458,10 @@ class OggFileType(FileType):
                 self.info = self._Info(fileobj)
                 self.tags = self._Tags(fileobj, self.info)
                 self.info._post_tags(fileobj)
-            except error, e:
-                raise self._Error, e, sys.exc_info()[2]
+            except error as e:
+                reraise(self._Error, e, sys.exc_info()[2])
             except EOFError:
-                raise self._Error, "no appropriate stream found"
+                raise self._Error("no appropriate stream found")
         finally:
             fileobj.close()
 
@@ -456,17 +470,19 @@ class OggFileType(FileType):
 
         If no filename is given, the one most recently loaded is used.
         """
+
         if filename is None:
             filename = self.filename
 
         self.tags.clear()
         fileobj = open(filename, "rb+")
         try:
-            try: self.tags._inject(fileobj)
-            except error, e:
-                raise self._Error, e, sys.exc_info()[2]
+            try:
+                self.tags._inject(fileobj)
+            except error as e:
+                reraise(self._Error, e, sys.exc_info()[2])
             except EOFError:
-                raise self._Error, "no appropriate stream found"
+                raise self._Error("no appropriate stream found")
         finally:
             fileobj.close()
 
@@ -475,14 +491,16 @@ class OggFileType(FileType):
 
         If no filename is given, the one most recently loaded is used.
         """
+
         if filename is None:
             filename = self.filename
         fileobj = open(filename, "rb+")
         try:
-            try: self.tags._inject(fileobj)
-            except error, e:
-                raise self._Error, e, sys.exc_info()[2]
+            try:
+                self.tags._inject(fileobj)
+            except error as e:
+                reraise(self._Error, e, sys.exc_info()[2])
             except EOFError:
-                raise self._Error, "no appropriate stream found"
+                raise self._Error("no appropriate stream found")
         finally:
             fileobj.close()
